
#   pedit.tcl      preference editor

set frame PEDIT
set title "Preference Editor"
set icontitle PEdit

set key ""
set preference ""
set value ""

set keys {}
set Current(preferences.filename)  "[file join $env(HOME) .midas preferences]"

proc update_preferences {} {
   global env Current key

   set lines [get_preference_file]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace $lines [first $range] [expr [second $range] + 1]]
       set new 0
   } else {
       set new 1
   }

   lappend lines "## $key"
   foreach item [array names Current] {
       lappend lines "$item $Current($item)"
   }
   lappend lines "##"

   put_preference_file $lines
   
   if {$new == 1} start
}

proc delete_key {} {
   global key

   set lines [get_preference_file]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace $lines [first $range] [expr [second $range] + 1]]
   }

   put_preference_file $lines
   
   start
}

proc click_key {v} {
    global key
    set key $v
    :modify KEYT :value $key
    load_preferences
}

proc click_preference {v} {
    global Current preference value
    
    set value $v
    set Current($preference) $value
    display_preferences
}

proc click_delete_preference {} {
    global Current preference
    
    unset Current($preference)
    display_preferences
}

proc load_preferences {} {
    global env key Current
    
    unset Current
    set Current(preferences.filename)  "[file join $env(HOME) .midas preferences]"

    set l {}
    foreach item [get_preferences $key] {
        set Current([first $item]) "[second $item]"
    }
    display_preferences
}

proc click_list {v} {
    global preference value
    
    set preference [first $v]
    set value [second $v]
    
    :modify PREFERENCE :value $preference
    :modify VALUE :value $value
}

proc display_preferences {} {
   global Current

    set l {}
    foreach item [array names Current] {
        lappend l "[format "%30s %50s" $item $Current($item)]"
    }
    :modify LIST :append F :strings [lsort -dictionary -index 0 $l] :see 0
}

proc click arg {
   global debug preference
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {

      KEYT -
      KEYB        {click_key $v}
      
      PREFERENCE  {set preference $v}
      VALUE       {click_preference $v}
      
      LIST        {click_list $v}

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc make-frame {} {
   global keys

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-name "[self icon-title]"
      
     :panel P1 :layout T
       :list LIST :rows 20  :cols 80 :exclusive T :scroll B
       
     :panel P2 :layout B :padx 10 :pady 10
      :layout H; :col-gap 20; :row-gap 5
       :text KEYT    :label Key  :w 16 
       :menubutton KEYB  :label "" :strings $keys
       :button SAVE      :label "Save Preferences"  :command update_preferences
       :button DELETE    :label "Delete Preference" :command click_delete_preference
       :button DELKEY    :label "Delete Key"        :command delete_key
      :next-row T
       :text PREFERENCE  :label "Preference" :w 20 :shifters T
       :text VALUE       :label "Value"      :w 40 :shifters T

   :show T
}

proc start args {
   global started keys
   
   set keys {}
   foreach item [get_preference_file] {
       if {[string length $item] > 3 && [string compare -length 3 $item "## "] == 0} then {
           lappend keys [string range $item 3 end]
       }
   }
       
   make-frame
   set started 1
insert-debug "$args"
   if {$args != ""} then {click_key [first $args]}
}


